#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       config.py
#       
#       Copyright 2009-2011 Sandro Mani <manisandro@gmail.com>
#       
#       This program is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This file contains the configuration class

import ConfigParser
import glob
import copy
import os
import sys
import gtk
import json
import gettext
import re
import enchant
_ = gettext.gettext

import dialogs

all_languages=[	["ara", u"\u0627\u0644\u0639\u0631\u0628\u064a\u0629", "ar_AE"],
				["bul", u"\u0431\u044a\u043b\u0433\u0430\u0440\u0441\u043a\u0438 \u0435\u0437\u0438\u043a", "bg_BG"],
				["cat", u"Catal\xe0", "ca_ES"],
				["ces", u"\u010cesky", "cs_CS"],
				["chi_sim", u"\u7b80\u4f53\u5b57", "zh_CN"],
				["chi_tra", u"\u7c21\u9ad4\u5b57", "zh_CN"],
				["dan-frak", u"Dansk (Frak)", "da_DK"],
				["dan", u"Dansk", "da_DK"],
				["deu-frak",u"Deutsch (Frak)","de_DE"],
				["deu-f",u"Deutsch (Frak)","de_DE"],
				["deu", u"Deutsch", "de_DE"],
				["ell", u"\u0395\u03bb\u03bb\u03b7\u03bd\u03b9\u03ba\u03ac", "el_GR"],
				["eng", u"English", "en_US"],
				["fin", u"Suomi", "fi_FI"],
				["fra", u"Fran\xe7ais", "fr_FR"],
				["heb", u"\u05e2\u05d1\u05e8\u05d9\u05ea", "he_IL"],
				["hrv", u"hrvatski", "hr_HR"],
				["hun", u"Magyar", "hu_HU"],
				["ind", u"Bahasa Indonesia", "id_ID"],
				["ita", u"Italiano", "it_IT"],
				["jpn", u"\u65e5\u672c\u8a9e", "ja_JP"],
				["kan", u"\u0c95\u0ca8\u0ccd\u0ca8\u0ca1", "kn_IN"],
				["kor", u"\ud55c\uad6d\ub9d0", "ko_KR"],
				["lav", u"Latvie\u0161u Valoda", "lv_LV"],
				["lit", u"Lietuvi\u0173 Kalba", "lt_LT"],
				["nld", u"Nederlands", "nl_NL"],
				["nor", u"Norsk", "no_NO"],
				["pol", u"Polski", "pl_PL"],
				["por", u"Portugu\xeas", "pt_PT"],
				["ron", u"Rom\xe2n\u0103", "ro_RO"],
				["rus", u"P\u0443\u0441\u0441\u043a\u0438\u0439 \u044f\u0437\u044b\u043a", "ru_RU"],
				["slk-frak", u"Sloven\u010dina", "sk_SK"],
				["slk", u"Sloven\u010dina", "sk_SK"],
				["slv", u"Sloven\u0161\u010dina", "sl_SI"],
				["spa", u"Espa\xf1ol", "es_ES"],
				["srp", u"\u0441\u0440\u043f\u0441\u043a\u0438 \u0458\u0435\u0437\u0438\u043a", "sr_RS"],
				["swe-frak", u"Svenska (Frak)", "sv_SE"],
				["swe", u"Svenska", "sv_SE"],
				["tgl", u"Wikang Tagalog", "tl_PH"],
				["tur", u"T\xfcrk\xe7e", "tr_TR"],
				["ukr", u"Y\u043a\u0440\u0430\u0457\u043d\u0441\u044c\u043a\u0430", "uk_UA"],
				["vie", u"Ti\u1ebfng Vi\u1ec7t", "vi_VN"] ]

dict_path=[os.path.join(os.sep,'usr','share','tesseract','tessdata'),os.path.join(os.sep,'usr','share','tesseract-ocr','tessdata'),os.path.join(os.sep,'usr','share','tessdata')]
tess_path=None

if sys.platform[0:3]=="win":	# Fix the dict dir, attempt to locate tesseract
	enchant.set_param("enchant.myspell.dictionary.path", os.environ['DICTDIR'])
	tess_path=glob.glob(os.path.join(os.environ["ProgramFiles"],'*tesseract*'))
	dict_path=[os.path.join(path,'tessdata') for path in tess_path]

class Config:
	def __init__(self,builder):
		self.opts={'paths':{
				'tess':{'id':'tess', 'desc':'Tesseract OCR', 'req':True, 'files':('tesseract','tesseract.exe'), 'basedir':tess_path, 'full':''},
				'dict':{'id':'dict', 'desc':'Tesseract dictionaries', 'req':True, 'files':('*.unicharset','*.traineddata'), 'basedir':dict_path, 'full':''}
			},
			'langs':[],
			'pane_orient':0,
			'pref_lang':None,
			'geometry':None,
			'spelldict_check':True,
			'update_check':True,
			'antialiasing':True,
			'custom_languages':[]
		}
		self.opts_bak=None
		
		# GUI variables
		self.builder=builder
		self.conf_dialog=builder.get_object("configdialog")
		self.conf_apply=builder.get_object("config_apply")
		self.spelldict_check=self.builder.get_object('spelldict_check')
		self.spelldict_check.connect('clicked', self.__checkbox_changed, 'spelldict_check')
		self.update_check=self.builder.get_object('update_check')
		self.update_check.connect('clicked', self.__checkbox_changed, 'update_check')
		self.check_antialiasing=self.builder.get_object('check_antialiasing')
		self.check_antialiasing.connect('clicked', self.__checkbox_changed, 'antialiasing')
		self.preflangbtn=self.builder.get_object("button_preflang")
		self.langmenu=None
		self.langitems=dict()
		self.preflangbtn.connect("clicked", lambda w: self.langmenu.popup(None, None, self.__menu_positioner, 1, gtk.get_current_event_time()))
		
		# Orientation combo
		self.paneorientcombo=gtk.combo_box_new_text()
		self.paneorientcombo.append_text(_("Vertical"))
		self.paneorientcombo.append_text(_("Horizontal"))
		builder.get_object("conf_entries").attach(self.paneorientcombo, 1, 2, 3, 4, yoptions=gtk.FILL)
		self.paneorientcombo.connect('changed', self.__set_paneorient)
		self.paneorientcombo.show()
		
		# Fill up dictionaries
		self.defaultdictsview=self.builder.get_object("treeview_dicts_default")
		self.defaultdictsstore=gtk.ListStore(str, str, str)
		self.defaultdictsview.set_model(self.defaultdictsstore)
		
		self.customdictsview=self.builder.get_object("treeview_dicts_custom")
		self.customdictsstore=gtk.ListStore(str, str, str)
		self.customdictsview.set_model(self.customdictsstore)
		self.customdictsview.get_selection().connect("changed", self.__customdict_entry_selected)
		self.remove_dict_btn=self.builder.get_object("button_dicts_remove")
		self.remove_dict_btn.set_sensitive(False)
		self.remove_dict_btn.connect("clicked", self.__customdict_entry_removed)
		self.add_dict_btn=self.builder.get_object("button_dicts_add")
		self.add_dict_btn.connect("clicked", self.__customdict_add)
		self.add_dict_cancel_btn=self.builder.get_object("button_dict_add_cancel")
		self.add_dict_cancel_btn.connect("clicked", self.__customdict_add_cancel)
		self.add_dict_ok_btn=self.builder.get_object("button_dict_add_ok")
		self.add_dict_ok_btn.connect("clicked", self.__customdict_add_ok)
		
		self.entry_prefix=self.builder.get_object("entry_dict_add_prefix")
		self.entry_name=self.builder.get_object("entry_dict_add_name")
		self.entry_code=self.builder.get_object("entry_dict_add_code")
		self.entry_prefix.connect("focus-in-event", self.__add_entry_focused)
		self.entry_name.connect("focus-in-event", self.__add_entry_focused)
		self.entry_code.connect("focus-in-event", self.__add_entry_focused)
		self.entry_prefix.connect("focus-out-event", self.__add_entry_blurred, _("Prefix"))
		self.entry_name.connect("focus-out-event", self.__add_entry_blurred, _("Name"))
		self.entry_code.connect("focus-out-event", self.__add_entry_blurred, _("ISO code"))
		self.__add_entry_blurred(self.entry_prefix, None, _("Prefix"))
		self.__add_entry_blurred(self.entry_name, None, _("Name"))
		self.__add_entry_blurred(self.entry_code, None, _("ISO code"))
		
		self.defaultdictsview.connect("realize", lambda w: self.__resize_columns(w))
		self.customdictsview.connect("realize", lambda w: self.__resize_columns(w))
		
		for lang in all_languages:
			self.defaultdictsstore.append(lang)
		
		for view in (self.defaultdictsview, self.customdictsview):
			for coldata in enumerate([_("Filename prefix"), _("Native name"), _("ISO 639-1 code")]):
				col=gtk.TreeViewColumn(coldata[1])
				col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
				cell=gtk.CellRendererText()
				col.pack_start(cell)
				col.set_attributes(cell, text=coldata[0])
				view.append_column(col)
		
		# Read configuration file
		if sys.platform[0:3]=="win":
			self.conf_path=os.path.join(os.environ["APPDATA"],'gimagereader')
		else:
			self.conf_path=os.path.join(os.path.expanduser("~"),'.config','gimagereader')
		self.conf_file=os.path.join(self.conf_path,'paths.cfg')
		confdata=None
		if os.path.isfile(self.conf_file):
			confdata=ConfigParser.RawConfigParser()
			confdata.read(self.conf_file)
		
		# Read / guess paths
		for id in self.opts['paths']:
			self.builder.get_object(id+'_path').connect("changed", self.__checkentry, id)
			self.__find_path(self.opts['paths'][id], confdata)
			
		self.__update_dictionaries(self.opts['paths']['dict']['basedir'])
		
		# Read prefered language
		if confdata and confdata.has_option("Prefs","language"):
			self.opts['pref_lang']=confdata.get("Prefs","language")
			if not self.opts['pref_lang'] in self.langitems:
				self.opts['pref_lang']=None
		
		# Read prefered orientation
		if confdata and confdata.has_option("Prefs","orientation"):
			try:
				pane_orient=int(confdata.get("Prefs","orientation"))
				if pane_orient==0 or pane_orient==1:
					self.opts['pane_orient']=pane_orient
			except:
				pass
			
		# Read other boolean preference:
		for opt in ("spelldict_check", "update_check", "antialiasing"):
			if confdata and confdata.has_option("Prefs",opt):
				try:
					self.opts[opt]=bool(int(confdata.get("Prefs",opt)))
				except:
					pass
		
		# Read stored geometry
		if confdata and confdata.has_option("Prefs","geometry"):
			try:
				self.opts['geometry']=map(int,json.loads(confdata.get("Prefs","geometry")))
			except:
				pass
		
		# Read custom languages
		try:
			self.opts['custom_languages']=json.loads(confdata.get("Prefs","custom_languages"))
			for lang in self.opts['custom_languages']:
				self.customdictsstore.append(lang)
		except:
			pass
		
		self.valid=self.__check_validity()
	
	def __check_validity(self):
		if not self.opts['langs']:
			return False
		for id in self.opts['paths']:
			if self.opts['paths'][id]['full']=='' and self.opts['paths'][id]['req']==True:
				return False
		return True
	
	def __find_path(self, path, confdata=None):
		paths=[]
		if confdata and confdata.has_option('Paths', path['id']) and confdata.get('Paths', path['id'])!='':
			paths.append(confdata.get('Paths', path['id']))
		if path['basedir']!=None:
			paths+=path['basedir']
		paths+=os.environ["PATH"].split(os.pathsep)
		for p in paths:
			for filepattern in path['files']:
				files=glob.glob(os.path.join(p,filepattern))
				if len(files)>0:
					path['basedir']=p
					path['full']=os.path.join(p, files[0])
					return
		path['basedir']=path['full']=''
	
	def __update_dictionaries(self, path):
		self.opts['langs']=[]
		self.langitems=dict()
		if path!='':
			for lang in all_languages+self.opts['custom_languages']:
				if os.path.isfile(os.path.join(path, lang[0]+'.unicharset')) or os.path.isfile(os.path.join(path, lang[0]+'.traineddata')):
					self.opts['langs'].append(lang)
		if not self.opts['langs']:
			return
		self.langmenu=gtk.Menu()
		alldicts=enchant.list_dicts()
		checkitem=None
		for lang in self.opts['langs']:
			item=gtk.MenuItem(lang[1])
			submenu=gtk.Menu()
			dicts=[ d[0] for d in alldicts if d[0][0:2]==lang[2][0:2] ]
			dicts.sort()
			if not dicts: dicts=[lang[2]]
			for d in dicts:
				checkitem=gtk.RadioMenuItem(checkitem, d, False)
				checkitem.connect("activate", self.__set_preflang, [lang[0], lang[1], d])
				self.langitems.update({d: checkitem})
				submenu.append(checkitem)
				if d==self.opts['pref_lang']: checkitem.set_active(True)
			item.set_submenu(submenu)
			self.langmenu.append(item)
		self.langmenu.show_all()
	
	def __set_preflang(self, widget, lang):
		self.preflangbtn.set_label("%s (%s)"%(lang[1], lang[2]))
		self.opts['pref_lang']=lang[2]
	
	def __menu_positioner(self, menu):
		x, y=self.conf_dialog.window.get_origin()
		x+=self.preflangbtn.allocation.x
		y+=self.preflangbtn.allocation.y
		return (x, y, False)
	
	def show_dialog(self):
		self.opts_bak=copy.deepcopy(self.opts)
		for id in self.opts['paths']:
			self.builder.get_object(id+'_path').set_text(self.opts['paths'][id]['basedir'])
			self.__checkentry(self.builder.get_object(id+'_path'), id)	# Assure icon consistency
		if self.opts['pref_lang'] in self.langitems:
			self.langitems[self.opts['pref_lang']].set_active(True)
		else:
			self.preflangbtn.set_label(_("(none)"))
		self.paneorientcombo.set_active(self.opts['pane_orient'])
		self.spelldict_check.set_active(self.opts['spelldict_check'])
		self.update_check.set_active(self.opts['update_check'])
		self.check_antialiasing.set_active(self.opts['antialiasing'])
		while True:
			response = self.conf_dialog.run()
			if response==gtk.RESPONSE_HELP:
				dialogs.help('Thirdparty')
			elif response != gtk.RESPONSE_APPLY:
				self.opts=self.opts_bak
				self.customdictsstore.clear()							# Ugly way...
				for lang in self.opts['custom_languages']:
					self.customdictsstore.append(lang)
				if self.valid==True or dialogs.question_dialog(_("Need valid paths to continue"),_("Correct the configuration? (Pressing 'No' will quit the program.)"), self.conf_dialog)!=1:
					self.conf_dialog.hide()
					break
			else:
				self.conf_dialog.hide()
				self.save_config()
				self.valid=True
				break
	
	def __checkbox_changed(self, widget, optkey):
		self.opts[optkey]=widget.get_active()
	
	def __checkentry(self, widget, id):
		value=widget.get_text()
		ok=False
		ico=self.builder.get_object(id+'_ok')
		ico.set_has_tooltip(False)
		if len(value)>0:
			for filepattern in self.opts['paths'][id]['files']:
				files=glob.glob(os.path.join(value,filepattern))
				if len(files)>0:
					ico.set_from_icon_name("gtk-yes","button")
					self.opts['paths'][id]['basedir']=value
					self.opts['paths'][id]['full']=os.path.join(value, files[0])
					ok=True
					break
		if id=='dict' and ok:
			self.__update_dictionaries(value)
		if not ok or (id=='dict' and not self.opts['langs']):
			if self.opts['paths'][id]['req']==True:
				ico.set_from_icon_name("gtk-no","button")
				ico.set_tooltip_text(_("This path is required for the program to work."))
			else:
				ico.set_from_icon_name("gtk-dialog-warning","button")
				ico.set_tooltip_text(_("Following features will not work:\n")+self.opts['paths'][id]['req'])
			ico.set_has_tooltip(True)
			self.opts['paths'][id]['basedir']=self.opts['paths'][id]['full']=''
		self.conf_apply.set_sensitive(self.__check_validity())
	
	def __set_paneorient(self, widget):
		if widget.get_active()!=-1:
			self.opts['pane_orient']=widget.get_active()
	
	### Dictionaries stuff ###
	def __resize_columns(self, widget):
		width=widget.allocation.width/len(widget.get_columns())
		for col in widget.get_columns():
			col.set_fixed_width(width)
		
	def __customdict_entry_selected(self, widget):
		selection = self.customdictsview.get_selection()
		model, iter = selection.get_selected()
		self.remove_dict_btn.set_sensitive((iter!=None and model.iter_is_valid(iter)))
	
	def __customdict_entry_removed(self, widget):
		selection = self.customdictsview.get_selection()
		model, iter = selection.get_selected()
		lang=self.customdictsstore.get_value(iter, 0)
		self.opts['custom_languages']=[ item for item in self.opts['custom_languages'] if not item[0]==lang ]
		self.customdictsstore.remove(iter)
	
	def __customdict_add(self, widget):
		self.builder.get_object("hbuttonbox_config_dicts").hide()
		self.builder.get_object("hbox_dicts_add_custom").show()
	
	def __customdict_add_cancel(self, widget):
		self.builder.get_object("hbuttonbox_config_dicts").show()
		self.builder.get_object("hbox_dicts_add_custom").hide()
		self.__add_entry_blurred(self.entry_prefix, None, _("Prefix"), True)
		self.__add_entry_blurred(self.entry_name, None, _("Name"), True)
		self.__add_entry_blurred(self.entry_code, None, _("ISO code"), True)
	
	def __add_entry_focused(self, widget, event):
		if widget.hintvisible==True:
			widget.hintvisible=False
			widget.set_text("")
		widget.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.get_color("text_color")))
		widget.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.get_color("base_color")))
	
	def __add_entry_blurred(self, widget, event, hinttext, force=False):
		if widget.get_text()=="" or force:
			widget.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse("#AAAAAA"))
			widget.hintvisible=True
			widget.set_text(hinttext)
	
	def __customdict_add_ok(self, widget):
		invalid=False
		for w, p in ((self.entry_prefix, r'^\w+$'), (self.entry_name, r'^.+$'), (self.entry_code, r'^[a-z]{2}_[A-Z]{2}$')):
			if w.hintvisible or not re.compile(p).match(w.get_text()):
				invalid=True
				w.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("#FF8080"))
		if not invalid:
			for w in (self.entry_prefix, self.entry_name, self.entry_code):
				w.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.get_color("base_color")))
			lang=[self.entry_prefix.get_text(), self.entry_name.get_text(), self.entry_code.get_text()]
			self.customdictsstore.append(lang)
			self.opts['custom_languages'].append(lang)
			self.__customdict_add_cancel(self.add_dict_cancel_btn)				# Reset 
			self.__checkentry(self.builder.get_object('dict_path'), 'dict')		# Refresh list / revalidate
	
	### External interface ###
	def get_color(self, key):
		scheme=gtk.settings_get_default().get_properties("gtk-color-scheme")[0]
		if scheme=='':	# Fallback
			scheme='tooltip_fg_color: #000000000000\nselected_bg_color: #31316a6ac5c5\ntooltip_bg_color: #f5f5f5f5b5b5\nbase_color: #ffffffffffff\nfg_color: #000000000000\ntext_color: #000000000000\nselected_fg_color: #ffffffffffff\nbg_color: #efefececdcdc\n'
		scheme=dict(item.split(": ") for item in scheme.split("\n") if item!='')
		return scheme[key]	
		
	def save_config(self):
		confdata=ConfigParser.RawConfigParser()
		confdata.add_section('Paths')
		for id in self.opts['paths']:
			confdata.set('Paths', id, self.opts['paths'][id]['basedir'])
		confdata.add_section('Prefs')
		confdata.set('Prefs', 'language', str(self.opts['pref_lang']))
		confdata.set('Prefs', 'orientation', str(self.opts['pane_orient']))
		confdata.set('Prefs', 'geometry', str(self.opts['geometry']))
		confdata.set('Prefs', 'spelldict_check', str(int(self.opts['spelldict_check'])))
		confdata.set('Prefs', 'update_check', str(int(self.opts['update_check'])))
		confdata.set('Prefs', 'antialiasing', str(int(self.opts['antialiasing'])))
		confdata.set('Prefs', 'custom_languages', json.JSONEncoder().encode((self.opts['custom_languages'])))
		try:
			if not os.path.isdir(self.conf_path):
				os.makedirs(self.conf_path)
			with open(self.conf_file, 'w') as configfile:
				confdata.write(configfile)
		except:
			pass
	
	def get_pane_orientation(self):
		return self.opts['pane_orient']
		
	def get_default_geometry(self):
		return self.opts['geometry']
		
	def set_default_geometry(self, pos, size):
		self.opts['geometry']=[pos[0],pos[1],size[0],size[1]];
	
	def get_dict_query(self):
		return self.opts['spelldict_check']
	
	def set_dict_query(self, boolean):
		self.opts['spelldict_check']=boolean
		
	def get_valid(self):
		return self.valid
	
	def get_languages(self):
		return self.opts['langs']
	
	def get_pref_lang(self):
		return self.opts['pref_lang']
	
	def get_update_check(self):
		return self.opts['update_check']
	
	def get_paths(self, pathids):
		paths=[]
		for id in pathids:
			paths.append(self.opts['paths'][id]['full'])
		return paths
	
	def get_antialiasing(self):
		return self.opts['antialiasing']
